import os
import random
import pandas as pd
from fastparquet import write
from tqdm import tqdm

PWD = os.path.dirname(__file__)
IN_DIR = os.path.join(PWD, "functions")

OUT_FILE = os.path.join(PWD, "extracted", "functions.pq")
OUT_SIZE = 500_000


def main():
    out_dir = os.path.dirname(OUT_FILE)
    if not os.path.isdir(out_dir):
        os.makedirs(out_dir)

    files = [f for f in os.listdir(IN_DIR) if f.endswith('.pq')]
    chosen: set[tuple[str, int]] = set()

    df = None

    with tqdm(desc="Sampling", total=OUT_SIZE) as p:
        while df is None or len(df.index) < OUT_SIZE:
            filename = random.choice(files)
            path = os.path.join(IN_DIR, filename)

            df_file = pd.read_parquet(path, engine='fastparquet')
            df_len = len(df_file.index)

            if df_len == 0:
                continue

            for _ in range(min(1000, df_len)):
                index = random.randrange(0, df_len)
                error_message = df_file.iloc[index]["error"]
                
                if error_message is not None and len(error_message) > 0:
                    continue
                
                source = df_file.iloc[index]["source"]

                if not source.isascii():
                    continue
                        
                choice = (filename, index)
                if choice not in chosen:
                    chosen.add(choice)
    
                    if df is None:
                        df = df_file
                    else:
                        df = pd.concat([df, df_file.iloc[[index], :]], axis=0, ignore_index=True)

                    if len(df.index) % 1000 == 0:
                        write(OUT_FILE, df, compression='GZIP')

                    p.update(1)

    # Fix success column, which is computed wrong in a previous iteration of extract.py
    df["success"] = df["error"].apply(lambda e: e is None or len(e) == 0)

    write(OUT_FILE, df, compression='GZIP')


if __name__ == "__main__":
    main()
